{% extends "base.html" %}
{% block title %}Create Project{% endblock %}
{% block content %}
<form action=create method=POST class="col-wide">
  <label for="name">Project name</label>
  <input type=text name=name id=name required>
  <label for="description">Description</label>
  <textarea name=description id=description required></textarea>
  <button>Create Project</button>
</form>
{% endblock %}